//! Keeping state
use super::*;
use tokio::{
    sync::{
	Semaphore,
	SemaphorePermit,
    },
};
use futures::{
    future::OptionFuture,
};

/// Contains state for a set of operations
#[derive(Debug)]
pub struct State
{
    cfg: config::Config,
    mtx: Option<Semaphore>,
}

/// Guard for operations inside state
#[derive(Debug)]
pub struct Permit<'a>(Option<SemaphorePermit<'a>>);

impl State
{
    /// Create a new state
    pub fn new(cfg: config::Config) -> Self
    {
	Self {
	    mtx: cfg.limit.map(|x| Semaphore::new(x.into())),
	    cfg,
	}
    }

    /// Acquire a permit for concurrent work, yielding the task if needed.
    pub async fn lock(&self) -> Permit<'_>
    {
	Permit(OptionFuture::from(self.mtx.as_ref().map(Semaphore::acquire)).await)
    }

    /// The config object
    #[inline] pub fn config(&self) -> &config::Config
    {
	&self.cfg
    }
}
